/* -*- mode: C -*-  */
/* 
   IGraph library.
   Copyright (C) 2007-2012  Gabor Csardi <csardi.gabor@gmail.com>
   334 Harvard street, Cambridge, MA 02139 USA
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc.,  51 Franklin Street, Fifth Floor, Boston, MA 
   02110-1301 USA

*/

#ifdef ATOMIC
#undef ATOMIC
#endif

#ifdef ATOMIC_IO
#undef ATOMIC_IO
#endif

#ifdef BASE
#undef BASE
#endif

#ifdef BASE_EPSILON
#undef BASE_EPSILON
#endif

#ifdef CONCAT2
#undef CONCAT2
#endif

#ifdef CONCAT2x
#undef CONCAT2x
#endif

#ifdef CONCAT3
#undef CONCAT3
#endif

#ifdef CONCAT3x
#undef CONCAT3x
#endif

#ifdef CONCAT4
#undef CONCAT4
#endif

#ifdef CONCAT4x
#undef CONCAT4x
#endif

#ifdef FP
#undef FP
#endif

#ifdef FUNCTION
#undef FUNCTION
#endif

#ifdef IN_FORMAT
#undef IN_FORMAT
#endif

#ifdef MULTIPLICITY
#undef MULTIPLICITY
#endif

#ifdef ONE
#undef ONE
#endif

#ifdef OUT_FORMAT
#undef OUT_FORMAT
#endif

#ifdef SHORT
#undef SHORT
#endif

#ifdef TYPE
#undef TYPE
#endif

#ifdef ZERO
#undef ZERO
#endif

#ifdef HEAPMORE
#undef HEAPMORE
#endif

#ifdef HEAPLESS
#undef HEAPLESS
#endif

#ifdef HEAPMOREEQ
#undef HEAPMOREEQ
#endif

#ifdef HEAPLESSEQ
#undef HEAPLESSEQ
#endif

#ifdef SUM
#undef SUM
#endif

#ifdef SQ
#undef SQ
#endif

#ifdef PROD
#undef PROD
#endif

#ifdef NOTORDERED
#undef NOTORDERED
#endif

#ifdef EQ
#undef EQ
#endif

#ifdef DIFF
#undef DIFF
#endif

#ifdef DIV
#undef DIV
#endif

#ifdef NOABS
#undef NOABS
#endif

#ifdef PRINTFUNC
#undef PRINTFUNC
#endif

#ifdef FPRINTFUNC
#undef PRINTFUNC
#endif

#ifdef UNSIGNED
#undef UNSIGNED
#endif
